/*
 *	FONT 
 *	TABSTOP=4
 */
#include "sony8fn.h"	/* SonyFn8x16rk */
#include "font.h"

#define ISKANJI1(c)	( ((c) > 0x80 && (c) < 0xa1) || ((c) > 0xdf && (c) < 0xfe) )


void FontPuts(const unsigned char *str, unsigned short fg, unsigned short bg,
	unsigned long pos)
{
	int kanji;
	unsigned long origX = pos & 0xffff;

	while (*str != '\0') {
		if (ISKANJI1(*str)) {
			kanji = (*str << 8) | str[1];
			GPU_sync();
			KanjiPutc(kanji, pos, fg, bg);

			str += 2;
			pos += 16;
		} else if (*str == '\r') {
			str ++;
			pos &= 0xffff0000;
			pos |= origX;
		} else if (*str == '\n') {
			str ++;
			pos += 0x00100000;
		} else {
			GPU_sync();
			KanaPutc(*str, pos, fg, bg);
			str ++;
			pos += 8;
		}
	}
	/* GPU_sync(); */
	return;
}

void KanaPutc(int c, unsigned long pos, unsigned short fg, unsigned short bg)
{
	unsigned char buf[8 * 16 * 2];
	int i, j;
	unsigned char *src, *dst;
	int bw;

	src = &SonyFn8x16rk[16 * c];
	dst = buf;
	for (i = 0 ; i < 16 ; i ++) {
		c = *src ++;
		for (j = 0 ; j < 8 ; j ++) {
			bw = ((c & 0x80) == 0x80);
			*(unsigned short *)dst = bw ? fg : bg;
			dst += 2;
			c <<= 1;
		}
	}
	mem2vram(pos & 0xffff, pos >> 16, 8, 16, buf);
	return;
}

/* TEXTWIN ֌W */
static const unsigned char *twFeedLine(const unsigned char *text, short width, int line);
static int twCalcLine(const unsigned char *text, short width);

/* RXgN^ */
void TwInit(TEXTWIN *pThis, RECT *pRect,
	unsigned short fg, unsigned short bg, const unsigned char *text)
{
	pThis->rect = *pRect;
	pThis->fg = fg; 
	pThis->bg = bg; 
	pThis->text = text; 

	pThis->curLine = 0;
	pThis->lastLine = twCalcLine(text, pRect->w >> 3);
}

void TwCls(TEXTWIN *pWin, short ox, short oy)
{
	RECT rect;

	rect = pWin->rect;
	rect.x += ox;
	rect.y += oy;
	FillRect(&rect, pWin->bg);
	GPU_sync();
}

void TwScroll(TEXTWIN *pWin, int bector)
{
	if (TwIsScrollable(pWin, bector) == false) return;
	pWin->curLine += bector;
}
void TwScrollByPage(TEXTWIN *pWin, int bector)
{
	int line;
	bector *= (pWin->rect.h >> 4);
	line = pWin->curLine + bector;
	if (line > pWin->lastLine)
		bector = pWin->lastLine - pWin->curLine;
	else if (line < 0) bector = - pWin->curLine;
	TwScroll(pWin, bector);
}

bool TwIsScrollable(TEXTWIN *pWin, int bector)
{
	int line = pWin->curLine + bector;
	if (line < 0) return false;
	if (line > pWin->lastLine) return false;

	return true;
}

void TwDraw(TEXTWIN *pWin, short ox, short oy)
{
	const unsigned char *text;
	short x, y;
	const short yMax = pWin->rect.h;
	const short xMax = pWin->rect.w;
	int kanji;
	short lineWidth = pWin->rect.w >> 3;
	int wordLen;
	int c;

	text = twFeedLine(pWin->text, lineWidth, pWin->curLine);

	ox += pWin->rect.x;
	oy += pWin->rect.y;
	for (y = 0 ; y + 15 < yMax ; y += 16) {
		for (x = 0 ; x < xMax ; ) {
			if (text == NULL || *text == '\0') {
				if (x + 8 > xMax) break;

				KanaPutc(' ', ((y + oy) << 16) | (x + ox), pWin->fg, pWin->bg);
				x += 8;
				continue;
			}

			if (*text == '\n') {
				text ++;
				while (x < xMax) {
					KanaPutc(' ', ((y + oy) << 16) | (x + ox), pWin->fg, pWin->bg);
					x += 8;
				}
				break;
			}

			if (*text == '\r') { text ++; continue; }

			if (ISKANJI1(*text)) {		/* KANJI */
				if (x + 16 > xMax) break;

				kanji = (*text << 8) | text[1];
				KanjiPutc(kanji, ((y + oy) << 16) | (x + ox), pWin->fg, pWin->bg);
				text += 2; x += 16;
			} else {						/* KANA */
				if (x + 8 > xMax) break;

				/* lineWidth ܂łɒP̐؂ڂAPꂪ邩ׂ */
				for (wordLen = 0 ; wordLen < lineWidth ; wordLen ++) {
					c = text[wordLen];
					if (c == ' ' || c < 0x20 || ISKANJI1(c)) break;
				}
				if (wordLen >= lineWidth || wordLen <= ((xMax - x) >> 3)) {
					KanaPutc(*text, ((y + oy) << 16) | (x + ox), pWin->fg, pWin->bg);
					text ++; x += 8;
				} else {
					while (x < xMax) {
						KanaPutc(' ', ((y + oy) << 16) | (x + ox), pWin->fg, pWin->bg);
						x += 8;
					}
					break;
				}
			}
		}
	}

	return;
}

/* \擪̕ւ̃|C^Ԃ */
static const unsigned char *twFeedLine(const unsigned char *text, short width, int line)
{
	short i;
	short wordLen;
	int c;

	if (text == NULL) return text;

	while (line > 0) {
		for (i = 0 ; i < width ; ) {
			if (*text == '\0') break;
			if (*text == '\n') { text ++; break; }

			if (*text == '\r') { text ++; i = 0; }
			else if (ISKANJI1(*text)) {
				if (i + 2 > width) break;
				text += 2; i += 2;
			} else {
				/* width ܂łɒP̐؂ڂAPꂪ邩ׂ */
				for (wordLen = 0 ; wordLen < width ; wordLen ++) {
					c = text[wordLen];
					if (c == ' ' || c < 0x20 || ISKANJI1(c)) break;
				}
				if (wordLen >= width || wordLen <= width - i) {
					text ++; i ++;
				} else break;
			}
		}
		line --;
	}
	return text;
}

/*
 *	text Sŉs邩
 */
static int twCalcLine(const unsigned char *text, short width)
{
	short i;
	short wordLen;
	int c;
	int line = 0;
/*	const unsigned char *orig = text; */

	if (text == NULL) return 0;

	for (line = 0 ; ; line ++) {
/*		printf("line=%d text=%d (%02x)\n", line, text - orig, *text & 0xff); */
		for (i = 0 ; i < width ; ) {
			if (*text == '\0') break;
			if (*text == '\n') { text ++; break; }

			if (*text == '\r') { text ++; i = 0; }
			else if (ISKANJI1(*text)) {
				if (i + 2 > width) break;
				text += 2; i += 2;
			} else {
				/* width ܂łɒP̐؂ڂAPꂪ邩ׂ */
				for (wordLen = 0 ; wordLen < width ; wordLen ++) {
					c = text[wordLen];
					if (c == ' ' || c < 0x20 || ISKANJI1(c)) break;
				}
				if (wordLen >= width || wordLen <= width - i) {
					text ++; i ++;
				} else break;
			}
		}
		if (*text == '\0') break;
	}
	return line;
}

